@@ -1,4 +1,4 @@
-autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, crash.retry=1, thin.sample = FALSE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200)){
+autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, raftery.options = list(), crash.retry=1, plots = TRUE, thin.sample = TRUE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200)){
 
 	
 	if(thin.sample==TRUE) thin.sample <- startsample
@@ -15,6 +15,36 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	
 	if(psrf.target <= 1) stop("psrf target must be greater than 1")
 	
+	if(!is.list(raftery.options)) stop("Options to raftery.diag must be provided as a named list")
+	
+	if(any(names(raftery.options)=="data")) warning("The 'data' argument specified in raftery.options was ignored")
+	
+	raftery.args <- formals(raftery.diag)
+	raftery.names <- names(raftery.args)
+
+	if(length(raftery.options) > 0){
+		if(is.null(names(raftery.options))) stop("Options to raftery.diag must be provided as a named list")
+		raft.opt.names <- names(raftery.options)
+		for(i in 1:length(raftery.options)){
+			if(any(raft.opt.names[i]==raftery.names)){
+				raftery.args[raft.opt.names[i]] <- raftery.options[i]
+			}else{
+				if(raft.opt.names[i]=="") stop("All arguments to raftery.diag must be named") else stop(paste(raft.opt.names[i], " is not a recognised argument to raftery.diag", sep=""))	
+			}	
+		}	
+
+	}
+	
+	success <- try({
+	raftery.args$data <- mcmc(1:startsample)
+	class(raftery.args) <- "list"
+	test <- do.call("raftery.diag", raftery.args)
+	})
+	
+	if(class(success)=="try-error") stop("The arguments specified for raftery.diag are not valid")
+	
+	if(test$resmatrix[1]=="Error") stop(paste("You need a startsample size of at least", test$resmatrix[2], "with the values of q, r and s specified for the raftery.options", sep=" "))
+	
 	if(class(silent.jags)=="list"){
 		killautocorr <- silent.jags$killautocorr
 		silent.jags = silent.jags$silent.jags
@@ -45,33 +75,44 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		adapt <- 0
 	}
 	
+	newlines <- if(silent.jags) "\n" else "\n\n"
 	
-	cat("\nAuto-run JAGS\n\nRunning a pilot chain...\n")
+	cat("\nAuto-run JAGS",newlines,"Running a pilot chain...\n",sep="")
 	pre.time <- Sys.time()
 	
 	
-	pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
-	if(any(pilot=="Unable to load coda files")) return(c("Error", "An error occured during the simulation"))
+	pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
+	if(any(pilot=="Unable to load coda files")){
+		cat("An error occured during the simulation\n\n")
+		return(c("Error", "An error occured during the simulation"))
+	}	
 	
-	
-	if(any(is.na(pilot))) return(c("Error", "An error occured during the simulation"))
-	if(class(pilot)=="character") return(c("Error", "An error occured during the simulation"))
+	if(any(is.na(pilot))){
+		cat("An error occured during the simulation\n\n")
+		return(c("Error", "An error occured during the simulation"))
+	}
+	if(class(pilot)=="character"){
+		cat("An error occured during the simulation\n\n")
+		return(c("Error", "An error occured during the simulation"))
+	}
 	
 	if(any(names(pilot)=="crash.end")){
 		repeat{
-			time.taken <- difftime(Sys.time(), pre.time, units="secs")
-			if(time.taken > max.time | crash.retry==0) return(c("Error", "The simulation was aborted due to crashes"))
-
-			cat("\nThe simulation crashed; retrying...\n\n")			
+			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
+			if(time.taken > max.time | crash.retry==0){
+				cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+				return(c("Error", "The simulation was aborted due to crashes"))
+			}
+			cat("\nThe simulation crashed; retrying...",newlines,sep="")			
 			crash.retry <- crash.retry - 1
 			oldpilot <- pilot
-			pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
+			pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
 			
 			if(!any(names(pilot)=="crash.end")) break
 		}
 	}
 	
-	time.taken <- difftime(Sys.time(), pre.time, units="secs")
+	time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 	cat("\n")
 	
 	
@@ -89,7 +130,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		cat("An error occured while calculating the autocorrelation; aborting simulation\n")
 		return(c("Error", "An error occured while calculating the autocorrelation"))
 	}
+	
+	options(show.error.messages = FALSE)
 	suppressWarnings(fsummary <- summary(final.mcmc))
+	options(show.error.messages = TRUE)
 	
 	convergence <- c(convergence, psrf.target=psrf.target)
 	class(convergence) <- "gelman.with.target"
@@ -128,6 +172,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				thrownaway <- 0
 			}
 			
+			pre.time <- Sys.time()-time.taken
 			additional <- pilot
 			
 			while(finishconv==FALSE){
@@ -140,29 +185,36 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				if(neededupdates > 0){
 					
 					oldadditional <- additional
-					additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=neededupdates, adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
+					additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=neededupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
 					pilot <- additional
 					
 					if(any(pilot=="Unable to load coda files")){
-						cat("\nThere was an error in the second simulation, possibly due to bad initial values or Random Number Seed values obtained from the first simulation.  Try monitoring less variables or using less chains.")
+						cat("\nThere was an error in the second simulation, possibly due to bad initial values or Random Number Seed values obtained from the first simulation.  You could try using fewer chains.")
 					}
 					
-					if(any(additional=="Unable to load coda files")) return(c("Error", "An error occured during the simulation"))
+					if(any(additional=="Unable to load coda files")){
+						cat("An error occured during the simulation\n\n")
+						return(c("Error", "An error occured during the simulation"))
+					}
 					if(any(names(additional)=="crash.end")){
 						repeat{
-							time.taken <- difftime(Sys.time(), pre.time, units="secs")
-							if(time.taken > max.time | crash.retry==0) return(c("Error", "The simulation was aborted due to crashes"))
-
+							time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
+							if(time.taken > max.time | crash.retry==0){
+								cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+								return(c("Error", "The simulation was aborted due to crashes"))
+							}
+							cat("\nThe simulation crashed; retrying...",newlines,sep="")
 							crash.retry <- crash.retry - 1
 							oldadd <- additional
-							additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = oldadditional$end.state, burnin=burnadapt, sample=(neededupdates), adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
+							additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = oldadditional$end.state, burnin=burnadapt, sample=(neededupdates), adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
 							
 							if(!any(names(additional)=="crash.end")) break
 						}
 					}
 					
-					final.mcmc <- window(combine.mcmc(list(final.mcmc, additional$mcmc)), start=neededupdates+1)
+					final.mcmc <- window(combine.mcmc(list(final.mcmc, additional$mcmc)), start=(((niter(final.mcmc)+niter(additional$mcmc))-startsample)+1))
 					
+					cat("Calculating the Gelman-Rubin statistic....\n")
 					thrownaway <- thrownaway+neededupdates
 					suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmc(final.mcmc, normalise=normalise.mcmc, warn=FALSE), transform=FALSE, autoburnin=TRUE), silent=TRUE))
 					if(class(success)=="try-error"){
@@ -174,8 +226,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 						cat("An error occured while calculating the autocorrelation; aborting simulation\n")
 						return(c("Error", "An error occured while calculating the autocorrelation"))
 					}
-				
+					
+					options(show.error.messages = FALSE)
 					suppressWarnings(fsummary <- summary(final.mcmc))
+					options(show.error.messages = TRUE)
 					
 					convergence <- c(convergence, psrf.target=psrf.target)
 					class(convergence) <- "gelman.with.target"
@@ -208,6 +262,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 							updatesdone <- updatesdone + neededupdates
 							updatesthrown <- updatesthrown + neededupdates
 						}
+						pre.time <- Sys.time()-time.taken
 					}else{
 						cat(paste("The Gelman-Rubin statistic is now below ", psrf.target, " for all parameters\n", sep=""))
 						finishconv <- TRUE
@@ -232,7 +287,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	
 	cat("\nCalculating the necessary sample length based on the Raftery and Lewis's diagnostic...\n")
 			
-	success <- try(raftery <- raftery.diag(normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE)))
+	success <- try({
+	raftery.args$data <- normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE)
+	class(raftery.args) <- "list"
+	raftery <- do.call("raftery.diag", raftery.args)
+	})
 	
 	#print(niter(final.mcmc))
 	#print(raftery)
@@ -241,10 +300,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	if(raftery[[1]]$resmatrix[1]=="error") return(c("Error", "An error occured while calculating the Raftery and Lewis diagnostic"))
 			
 	
-	# to correct for monitoring arrays:
-	monitor <- dimnames(raftery[[1]]$resmatrix)[[1]]
+	# to correct for monitoring arrays and non-stochastic nodes:
+	newmonitor <- dimnames(raftery[[1]]$resmatrix)[[1]]
 	
-	dependance = burnin = sample <- matrix(ncol=n.chains, nrow=length(monitor), dimnames=list(dimnames(raftery[[1]]$resmatrix)[[1]], 1:n.chains))
+	dependance = burnin = sample <- matrix(ncol=n.chains, nrow=length(newmonitor), dimnames=list(dimnames(raftery[[1]]$resmatrix)[[1]], 1:n.chains))
 	
 	for(i in 1:n.chains){	
 		dependance[,i] <- raftery[[i]]$resmatrix[,"I"]
@@ -269,34 +328,45 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				
 			}
 		}else{
-			cat("The model would need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((time.taken*moreupdates/(startsample+startburnin))), ", taking the total simulation length to over ", timestring(max.time), ".\n", sep="")
-			stop <- TRUE
+			cat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((time.taken*moreupdates/(startsample+startburnin))), ".\n", sep="")
+			#cat("The model would need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((time.taken*moreupdates/(startsample+startburnin))), ", taking the total simulation length to over ", timestring(max.time), ".\n", sep="")  NOW NO LONGER ADDING THIS TO TIME LIMIT
+			#stop <- TRUE
 			if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)) stop <- !ask("Continue with the simulation?")
 			if(stop){
 				cat("Simulation aborted\n\n")
 				return(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), req.burnin=max(burnin), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr))
 			}
-			
 		}
+		pre.time <- Sys.time()-time.taken
 		cat("\n")
-		additional <- run.jags(data=data, model=model, monitor=monitor,  n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
-		if(any(additional=="Unable to load coda files")) return(c("Error", "An error occured during the simulation"))
+		additional <- run.jags(data=data, model=model, monitor=monitor,  n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
+		if(any(additional=="Unable to load coda files")){
+			cat("An error occured during the simulation\n\n")
+			return(c("Error", "An error occured during the simulation"))
+		}
 		
 		if(any(names(additional)=="crash.end")){
 			repeat{
-				time.taken <- difftime(Sys.time(), pre.time, units="secs")
-				if(time.taken > max.time | crash.retry==0) return(c("Error", "The simulation was aborted due to crashes"))
-				cat("\nThe simulation crashed; retrying...\n\n")			
+				time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
+				if(time.taken > max.time | crash.retry==0){
+					cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+					return(c("Error", "The simulation was aborted due to crashes"))
+				}
+				cat("\nThe simulation crashed; retrying...",newlines,sep="")			
 				crash.retry <- crash.retry - 1
 				oldadd <- additional
-				additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, check.conv=FALSE)
-				if(any(additional=="Unable to load coda files")) return(c("Error", "An error occured during the simulation"))
+				additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE)
+				if(any(additional=="Unable to load coda files")){
+					cat("An error occured during the simulation\n\n")
+					return(c("Error", "An error occured during the simulation"))
+				}
 				if(!any(names(additional)=="crash.end")) break
 			}
 		}
 		
 		final.mcmc <- combine.mcmc(list(final.mcmc, additional$mcmc))
 		
+		
 	}else{
 		additional <- pilot # for end.state at the end, pilot is output of either first sim or extended convergence sim
 	}	
@@ -317,7 +387,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		cat("An error occured while calculating the autocorrelation; aborting simulation\n")
 		return(c("Error", "An error occured while calculating the autocorrelation"))
 	}
+	
+	
+	options(show.error.messages = FALSE)
 	suppressWarnings(fsummary <- summary(final.mcmc))
+	options(show.error.messages = TRUE)
 	
 	convergence <- c(convergence, psrf.target=psrf.target)
 	class(convergence) <- "gelman.with.target"
@@ -336,7 +410,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		}	
 	}
 
-	if(unconverged > 0){
+	if(unconverged > 0 & !killautocorr){
 		cat(paste("WARNING:  The Gelman-Rubin statistic for ", unconverged, " parameter(s) are above ", psrf.target, " for the final chains (multi-variate psrf = ", as.numeric(round(convergence$mpsrf, digits=3)), ").  The chains may have fallen out of convergence.  It is important to assess convergence manually before relying on inference from these chains.\n", sep=""))
 	}
 	
@@ -363,17 +437,52 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		}else{
 			cat("Autocorrelation could not be calculated for these chains\n")
 		}
+		unused <- normalise.mcmc(final.mcmc, normalise=FALSE, warn="warning")  # SO THAT WARNING OF NON STOCHASTICITY IS PRINTED
 	}
 	
-	suppressWarnings(final.mcmc <- window(final.mcmc, thin=max(1, floor(n.iters/thin.sample))))
-	
-	unused <- normalise.mcmc(final.mcmc, normalise=FALSE, warn="warning")  # SO THAT WARNING OF NON STOCHASTICITY IS PRINTED
+	final.mcmc <- combine.mcmc(final.mcmc, return.samples=thin.sample)
 
+
+	if(plots==TRUE){
+		plot1 = plot2 = vector('list', length=length(varnames(final.mcmc)))
+		names(plot1) = names(plot2) <- varnames(final.mcmc)
+		thinned.mcmc <- combine.mcmc(list(final.mcmc), return.samples=1000)
+
+		startdev <- dev.list()
+
+		#a <- dev.new()
+		#if(options("device")$device=="x11") x11()
+	
+		for(i in 1:length(varnames(final.mcmc))){
+	
+			plotdata <- thinned.mcmc[,c(i,i)]
+			varnames(plotdata)[1] <- ""		
+			plot1[[i]] <- xyplot(plotdata, layout=c(1,1), ylab="Value", xlab="Iteration", lattice.options=list(par.settings=plot.new()))
+			class(plot1[[i]]) <- "plotindpages"
+			plot2[[i]] <- densityplot(plotdata, layout=c(1,1), ylab="Density", xlab="Value")
+			class(plot2[[i]]) <- "plotindpages"
+	
+		}
+	
+		if(!is.null(startdev)){
+			for(i in dev.list()){
+				if(!any(startdev==i)) dev.off(i)
+			}
+		}else{
+			a <- dev.new()
+			while(!is.null(dev.list())){
+				dev.off()
+			}
+		}
+	}else{
+		plot1 = plot2 <- "Plots not produced when plots==FALSE"
+	}
+	
 	cat("Auto-run JAGS complete.\n\n")
 	if(killautocorr==FALSE) cat("*PLEASE NOTE:  THIS SOFTWARE IS INTENDED FOR EDUCATIONAL PURPOSES ONLY*\n*YOU SHOULD ASSESS CONVERGENCE AND AUTOCORRELATION MANUALLY BEFORE RELYING ON RESULTS PROVIDED*\n\n")
 	
 	
-	return(list(mcmc=final.mcmc, end.state=additional$end.state, req.samples=max(sample), req.burnin=max(burnin), samples.to.conv=updatesthrown, summary=fsummary, psrf=convergence, autocorr=autocorrelation))
+	return(list(mcmc=final.mcmc, end.state=additional$end.state, req.samples=max(sample), req.burnin=max(burnin), samples.to.conv=updatesthrown, summary=fsummary, psrf=convergence, autocorr=autocorrelation, trace=plot1, density=plot2))
 }
 	
 autorun.JAGS <- autorun.jags
